;; Generated package description from rust-mode.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "rust-mode" "1.0.6.0.20260521.74503" "A major-mode for editing Rust source code" '((emacs "25.1")) :commit "ed401a65743359b8de11ee9ced0e1da39946cefd" :authors '(("Mozilla" . "rust-mode@noreply.github.com")) :maintainer '("Mozilla" . "rust-mode@noreply.github.com") :keywords '("languages") :url "https://github.com/rust-lang/rust-mode")
